@@ -4,7 +4,7 @@
  * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2015 Pascal Buchbinder
+ * Copyright (C) 2007-2016 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +23,7 @@
  *
  */
 
-static const char revision[] = "$Id: qs_util.c,v 1.18 2015/05/04 20:24:59 pbuchbinder Exp $";
+static const char revision[] = "$Id: qs_util.c,v 1.21 2016/01/08 15:32:56 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <pthread.h>
@@ -307,7 +307,7 @@ void qs_setuid(const char *username, const char *cmd) {
     struct passwd *pwd = getpwnam(username);
     uid_t uid, gid;
     if(pwd == NULL) {
-      fprintf(stderr, "[%s] unknown user id '%s': %s", cmd, username, strerror(errno));
+      fprintf(stderr, "[%s] failed to switch user: unknown user id '%s'\n", cmd, username);
       exit(1);
     }
     uid = pwd->pw_uid;
@@ -315,11 +315,11 @@ void qs_setuid(const char *username, const char *cmd) {
     setgid(gid);
     setuid(uid);
     if(getuid() != uid) {
-      fprintf(stderr, "[%s] setuid failed (%s,%d)", cmd, username, uid);
+      fprintf(stderr, "[%s] setuid failed (%s,%d)\n", cmd, username, uid);
       exit(1);
     }
     if(getgid() != gid) {
-      fprintf(stderr, "[%s] setgid failed (%d)", cmd, gid);
+      fprintf(stderr, "[%s] setgid failed (%d)\n", cmd, gid);
       exit(1);
     }
   }
