@@ -5,7 +5,7 @@
  *
  * See http://opensource.adnovum.ch/mod_qos/ for further details.
  *
- * Copyright (C) 2011-2014 Pascal Buchbinder
+ * Copyright (C) 2011-2015 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,7 +27,7 @@
  *
  */
 
-static const char revision[] = "$Id: qsexec.c,v 1.21 2014/03/25 11:57:09 pbuchbinder Exp $";
+static const char revision[] = "$Id: qsexec.c,v 1.26 2015/05/04 20:24:59 pbuchbinder Exp $";
 
 /* system */
 #include <stdio.h>
@@ -37,8 +37,6 @@ static const char revision[] = "$Id: qsexec.c,v 1.21 2014/03/25 11:57:09 pbuchbi
 #include <unistd.h>
 #include <time.h>
 
-#include <pwd.h>
-
 /* apr */
 #include <pcre.h>
 #include <apr.h>
@@ -259,7 +257,7 @@ int qs_regexec(pcre *preg, const char *string,
 int main(int argc, const char * const argv[]) {
   const char *username = NULL;
   int nr = 0;
-  char line[32768];
+  char *line = calloc(1, MAX_LINE_BUFFER+1);
   apr_pool_t *pool;
   char *cmd = strrchr(argv[0], '/');
   const char *command = NULL;
@@ -324,13 +322,15 @@ int main(int argc, const char * const argv[]) {
       }
     } else if(argc >= 1 && strcmp(*argv,"-p") == 0) {
       pass = 1;
-    } else if(argc >= 1 && strcmp(*argv,"-h") == 0) {
+    } else if(strcmp(*argv,"-h") == 0) {
       usage(cmd, 0);
-    } else if(argc >= 1 && strcmp(*argv,"-?") == 0) {
+    } else if(strcmp(*argv,"-?") == 0) {
       usage(cmd, 0);
-    } else if(argc >= 1 && strcmp(*argv,"-help") == 0) {
+    } else if(strcmp(*argv,"-help") == 0) {
       usage(cmd, 0);
-    } else if(argc >= 1 && strcmp(*argv,"--man") == 0) {
+    } else if(strcmp(*argv,"--help") == 0) {
+      usage(cmd, 0);
+    } else if(strcmp(*argv,"--man") == 0) {
       usage(cmd, 1);
     } else {
       command = *argv;
@@ -343,26 +343,7 @@ int main(int argc, const char * const argv[]) {
     usage(cmd, 0);
   }
 
-  if(username && getuid() == 0) {
-    struct passwd *pwd = getpwnam(username);
-    uid_t uid, gid;
-    if(pwd == NULL) {
-      fprintf(stderr,"[%s]: ERROR, unknown user id %s\n", cmd, username);
-      exit(1);
-    }
-    uid = pwd->pw_uid;
-    gid = pwd->pw_gid;
-    setgid(gid);
-    setuid(uid);
-    if(getuid() != uid) {
-      fprintf(stderr,"[%s]: ERROR, setuid failed (%s,%d)\n", cmd, username, uid);
-      exit(1);
-    }
-    if(getgid() != gid) {
-      fprintf(stderr,"[%s]: ERROR, setgid failed (%d)\n", cmd, gid);
-      exit(1);
-    }
-  }
+  qs_setuid(username, cmd);
 
   preg = pcre_compile(pattern, PCRE_DOTALL, &errptr, &erroffset, NULL);
   if(!preg) {
@@ -379,7 +360,7 @@ int main(int argc, const char * const argv[]) {
     }
   }
 
-  while(fgets(line, sizeof(line), stdin) != NULL) {
+  while(fgets(line, MAX_LINE_BUFFER, stdin) != NULL) {
     nr++;
     if(pass) {
       printf("%s", line);
