@@ -28,7 +28,7 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c,v 1.96 2015/01/05 17:35:59 pbuchbinder Exp $";
+static const char revision[] = "$Id: qslog.c,v 1.98 2015/09/21 20:47:13 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -1928,7 +1928,7 @@ static stat_rec_t *loadRule(apr_pool_t *pool, const char *confFile) {
   stat_rec_t *prev = NULL;
   stat_rec_t *next = NULL;
   if(file == NULL) {
-    qerror("could not open file for writing '%s': ", confFile, strerror(errno));
+    qerror("could not read file '%s': ", confFile, strerror(errno));
     exit(1);
   }
   while(!qs_getLinef(line, sizeof(line), file)) {
@@ -2275,26 +2275,7 @@ int main(int argc, const char *const argv[]) {
   /* requires at least a format string */
   if(config == NULL) usage(cmd, 0);
 
-  if(username && getuid() == 0) {
-    struct passwd *pwd = getpwnam(username);
-    uid_t uid, gid;
-    if(pwd == NULL) {
-      qerror("unknown user id '%s': %s", username, strerror(errno));
-      exit(1);
-    }
-    uid = pwd->pw_uid;
-    gid = pwd->pw_gid;
-    setgid(gid);
-    setuid(uid);
-    if(getuid() != uid) {
-      qerror("setuid failed (%s,%d)", username, uid);
-      exit(1);
-    }
-    if(getgid() != gid) {
-      qerror("setgid failed (%d)", gid);
-      exit(1);
-    }
-  }
+  qs_setuid(username, cmd);
 
   if(file) {
     m_f = fopen(file, "a+"); 
