@@ -6,7 +6,7 @@
  * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2010-2015 Pascal Buchbinder
+ * Copyright (C) 2010-2016 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,7 +28,7 @@
  *
  */
 
-static const char revision[] = "$Id: qssign.c,v 1.35 2015/05/04 20:24:59 pbuchbinder Exp $";
+static const char revision[] = "$Id: qssign.c,v 1.43 2016/01/08 15:32:56 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <unistd.h>
@@ -41,7 +41,11 @@ static const char revision[] = "$Id: qssign.c,v 1.35 2015/05/04 20:24:59 pbuchbi
 #include <openssl/evp.h>
 #include <openssl/hmac.h>
 
+/* PCRE */
+#include <pcre.h>
+
 /* apr/apr-util */
+#define QS_USEAPR 1
 #include <apr.h>
 #include <apr_base64.h>
 #include <apr_pools.h>
@@ -51,6 +55,7 @@ static const char revision[] = "$Id: qssign.c,v 1.35 2015/05/04 20:24:59 pbuchbi
 #include <apr_time.h>
 
 #include "qs_util.h"
+#include "qs_apo.h"
 
 #define SEQDIG "12"
 
@@ -62,6 +67,8 @@ static int  m_logend = 0;
 static void (*m_end)(const char *) = NULL;
 static int m_end_pos = 0;
 static const char *m_sec = NULL;
+static const EVP_MD *m_evp;
+static const pcre *m_filter = NULL;
 
 typedef struct {
   const char* fmt;
@@ -110,7 +117,7 @@ static void qs_write(char *line, int line_size, const char *sec, int sec_len) {
   char *m;
   int data_len;
   sprintf(&line[strlen(line)], " %."SEQDIG"ld", m_nr);
-  HMAC_Init(&ctx, sec, sec_len, EVP_sha1());
+  HMAC_Init(&ctx, sec, sec_len, m_evp);
   HMAC_Update(&ctx, (const unsigned char *)line, strlen(line));
   HMAC_Final(&ctx, data, &len);
   m = calloc(1, apr_base64_encode_len(len) + 1);
@@ -383,7 +390,12 @@ static void qs_sign(const char *sec) {
     if(m_logend && (m_end == NULL)) {
       qs_set_format(line);
     }
-    qs_write(line, line_size, sec, sec_len);
+    if(pcre_exec(m_filter, NULL, line, line_size, 0, 0, NULL, 0) >= 0) {
+      printf("%s\n", line);
+      fflush(stdout);
+    } else {
+      qs_write(line, line_size, sec, sec_len);
+    }
   }
   return;
 }
@@ -397,6 +409,8 @@ static long qs_verify(const char *sec) {
   int line_size = MAX_LINE_BUFFER;
   int line_len;
   m_nr = -1; // sequence number
+  long nr_alt = -1;
+  long nr_alt_lnr = -1;
   while(fgets(line, line_size, stdin) != NULL) {
     int valid = 0;
     long ns = 0;
@@ -422,7 +436,7 @@ static long qs_verify(const char *sec) {
       sig[0] = '\0';
       sig++;
       /* verify hmac */
-      HMAC_Init(&ctx, sec, sec_len, EVP_sha1());
+      HMAC_Init(&ctx, sec, sec_len, m_evp);
       HMAC_Update(&ctx, (const unsigned char *)line, strlen(line));
       HMAC_Final(&ctx, data, &len);
       m = calloc(1, apr_base64_encode_len(len) + 1);
@@ -431,6 +445,10 @@ static long qs_verify(const char *sec) {
       if(strcmp(m, sig) != 0) {
 	err++;
 	fprintf(stderr, "ERROR on line %ld: invalid signature\n", lnr);
+	/* message may be modified/currupt or inserted: next line may have
+	   the next sequence number (modified) or the same (inserted) */
+	nr_alt = m_nr + 1;
+	nr_alt_lnr = lnr + 1;
       } else {
 	valid = 1;
       }
@@ -443,6 +461,15 @@ static long qs_verify(const char *sec) {
 	fprintf(stderr, "ERROR on line %ld: invalid sequence\n", lnr);
       } else {
 	if(m_nr != -1) {
+	  if(lnr == nr_alt_lnr) {
+	    // last line was modfied
+	    if(m_nr != ns) {
+	      // and therefore, we also accept the next seqence number
+	      m_nr = nr_alt;
+	    }
+	    nr_alt = -1;
+	    nr_alt_lnr = -1;
+	  }
 	  if(m_nr != ns) {
 	    if(ns == 1) {
 	      if(!end_seen) {
@@ -456,7 +483,7 @@ static long qs_verify(const char *sec) {
 	    }
 	  }
 	} else if(m_logend) {
-	  // log should (if not rotated) with message 0
+	  // log should (if not rotated) start with message 0
 	  if(ns != 1) {
 	    fprintf(stderr, "NOTICE: log starts with sequence %."SEQDIG"ld, log rotation?"
 		    " (expect %."SEQDIG"d)\n", ns, 1);
@@ -489,83 +516,6 @@ static long qs_verify(const char *sec) {
   return err;
 }
 
-static void qs_failedexec(const char *msg, const char *cmd, apr_status_t status) {
-  char buf[MAX_LINE];
-  apr_strerror(status, buf, sizeof(buf));
-  fprintf(stderr, "ERROR %s '%s': '%s'\n", msg, cmd, buf);
-  exit(1);
-}
-
-static apr_table_t *qs_args(apr_pool_t *pool, const char *line) {
-  char *last = apr_pstrdup(pool, line);
-  apr_table_t* table = apr_table_make(pool, 10);
-  char *val;
-  while((val = apr_strtok(NULL, " ", &last))) {
-    apr_table_addn(table, val, "");
-  }
-  return table;
-}
-
-static char *qs_readpwd(apr_pool_t *pool, const char *prg) {
-  apr_status_t status;
-  apr_proc_t proc;
-  const char **args;
-  apr_table_entry_t *entry;
-  char *last;
-  char *copy = apr_pstrdup(pool, prg);
-  char *cmd = apr_strtok(copy, " ", &last);
-  apr_table_t *a = qs_args(pool, prg);
-  int i;
-  apr_procattr_t *attr;
-  apr_size_t len = MAX_LINE;
-  char *buf = apr_pcalloc(pool, len);
-
-  args = apr_pcalloc(pool, (apr_table_elts(a)->nelts + 1) * sizeof(const char *));
-  entry = (apr_table_entry_t *) apr_table_elts(a)->elts;
-  for(i = 0; i < apr_table_elts(a)->nelts; i++) {
-    args[i] = entry[i].key;
-  }
-  args[i] = NULL;
-
-  if(cmd == NULL) {
-    qs_failedexec("can't read password, invalid executable", prg, APR_EGENERAL);
-  }
-  if((status = apr_procattr_create(&attr, pool)) != APR_SUCCESS) {
-    qs_failedexec("while reading password from executable", prg, status);
-  }
-  if((status = apr_procattr_cmdtype_set(attr, APR_PROGRAM_PATH)) != APR_SUCCESS) {
-    qs_failedexec("while reading password from executable", prg, status);
-  }
-  if((status = apr_procattr_detach_set(attr, 0)) != APR_SUCCESS) {
-    qs_failedexec("while reading password from executable", prg, status);
-  }
-  if((status = apr_procattr_io_set(attr, APR_FULL_BLOCK, APR_FULL_BLOCK, APR_NO_PIPE)) != APR_SUCCESS) {
-    qs_failedexec("while reading password from executable", prg, status);
-  }
-  if((status = apr_proc_create(&proc, cmd, args, NULL, attr, pool)) != APR_SUCCESS) {
-    qs_failedexec("could not execute program", prg, status);
-  } else {
-    char *e;
-    status = apr_proc_wait(&proc, NULL, NULL, APR_WAIT);
-    if(status != APR_CHILD_DONE && status != APR_SUCCESS) {
-      qs_failedexec("while reading password from executable", prg, status);
-    }
-    status = apr_file_read(proc.out, buf, &len);
-    if(status != APR_SUCCESS) {
-      qs_failedexec("failed to read password from program", prg, status);
-    }
-    e = buf;
-    while(e && e[0]) {
-      if((e[0] == LF) || (e[0] == CR)) {
-	e[0] = '\0';
-      } else {
-	e++;
-      }
-    }
-  }
-  return buf;
-}
-
 static void usage(char *cmd, int man) {
   if(man) {
     //.TH [name of program] [section number] [center footer] [left footer] [center header]
@@ -581,7 +531,7 @@ static void usage(char *cmd, int man) {
   if(man) {
     printf(".SH SYNOPSIS\n");
   }
-  qs_man_print(man, "%s%s -s|S <secret> [-e] [-v] [-u <name>]\n", man ? "" : "Usage: ", cmd);
+  qs_man_print(man, "%s%s -s|S <secret> [-e] [-v] [-u <name>] [-f <regex>] [-a 'sha1'|sha256']\n", man ? "" : "Usage: ", cmd);
   printf("\n");
   if(man) {
     printf(".SH DESCRIPTION\n");
@@ -589,7 +539,8 @@ static void usage(char *cmd, int man) {
     printf("Summary\n");
   }
   qs_man_print(man, "%s is a log data integrity check tool. It reads log data\n", cmd);
-  qs_man_print(man, "from stdin (pipe) and writes the signed data to stdout.\n");
+  qs_man_print(man, "from stdin (pipe) and writes the data to stdout adding a sequence\n");
+  qs_man_print(man, "number and signatur to ever log line.\n");
   printf("\n");
   if(man) {
     printf(".SH OPTIONS\n");
@@ -616,6 +567,15 @@ static void usage(char *cmd, int man) {
   qs_man_print(man, "  -u <name>\n");
   if(man) printf("\n");
   qs_man_print(man, "     Becomes another user, e.g. www-data.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -f <regex>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Filter pattern (case sensitive regular expression) for messages\n");
+  qs_man_print(man, "     which do not need to be signed.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -a 'sha1'|'sha256'\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Specifes the algorithm to use. Default is sha1.\n");
   printf("\n");
   if(man) {
     printf(".SH EXAMPLE\n");
@@ -655,6 +615,7 @@ int main(int argc, const char * const argv[]) {
   int verify = 0;
   char *cmd = strrchr(argv[0], '/');
   const char *username = NULL;
+  const char *filter = NULL;
   if(cmd == NULL) {
     cmd = (char *)argv[0];
   } else {
@@ -662,7 +623,7 @@ int main(int argc, const char * const argv[]) {
   }
   apr_app_initialize(&argc, &argv, NULL);
   apr_pool_create(&pool, NULL);
-
+  m_evp = EVP_sha1();
   argc--;
   argv++;
   while(argc >= 1) {
@@ -682,6 +643,21 @@ int main(int argc, const char * const argv[]) {
       if (--argc >= 1) {
         username = *(++argv);
       }
+    } else if(strcmp(*argv,"-f") == 0) { /* filter */
+      if (--argc >= 1) {
+        filter = *(++argv);
+      }
+    } else if(strcmp(*argv,"-a") == 0) { /* set alg */
+      if (--argc >= 1) {
+        const char *alg = *(++argv);
+	if(strcasecmp(alg, "SHA256") == 0) {
+	  m_evp = EVP_sha256();
+	} else if(strcasecmp(alg, "SHA1") != 0) {
+	  m_evp = NULL;
+	}
+      } else {
+	m_evp = NULL;
+      }
     } else if(strcmp(*argv,"-?") == 0) {
       usage(cmd, 0);
     } else if(strcmp(*argv,"-help") == 0) {
@@ -695,6 +671,21 @@ int main(int argc, const char * const argv[]) {
     argv++;
   }
 
+  if(filter != NULL) {
+    const char *errptr = NULL;
+    int erroffset;
+    m_filter = pcre_compile(filter, 0, &errptr, &erroffset, NULL);
+    if(m_filter == NULL) {
+      fprintf(stderr, "failed to compile filter pattern <%s> at position %d,"
+            " reason: %s\n", filter, erroffset, errptr);
+      exit(1);
+    }
+  }
+
+  if(m_evp == NULL) {
+    usage(cmd, 0);
+  }
+    
   if(m_sec == NULL) {
     usage(cmd, 0);
   }
